@@ -7,17 +7,20 @@ import traceback
 import typing as ty
 
 import discord
+import tabulate
+from loguru import logger
 
 from . import FluxCog
 from .. import CommandEvent, utils
 from ..auth import Auth, AuthList, Record
 from ..command import Response
-from ..context import GuildMessageCtx, ManualAuthCtx, ManualAuthorCtx
-from ..errors import CommandError
+from ..context import ManualAuthCtx, ManualAuthorCtx, CommandCtx, GuildMessageCtx
+from ..errors import CommandError, BotMissingPermissions
 
 if ty.TYPE_CHECKING:
-   from ..context import CommandCtx, GuildAwareCtx
+   from ..context import GuildAwareCtx
    from ..command import Command
+   from ..types_ import GuildCommandCtx
 
 
 class Builtins(FluxCog):
@@ -31,13 +34,13 @@ class Builtins(FluxCog):
    }
 
    def load(self):
-      def parse_auth_id(ctx: GuildAwareCtx, type_: str, target_: str) -> int:
+      async def parse_auth_id(ctx: GuildAwareCtx, type_: str, target_: str) -> int:
          if type_ == "member":
             ids_ = utils.find_mentions(target_)
 
             if not ids_:
                raise CommandError(f"No member ID found in {target_}")
-            if not ctx.guild.get_member(ids_[0]):
+            if not await utils.get_or_fetch_member(ctx.guild, ids_[0]):
                raise CommandError(f"No member found with ID {ids_[0]}")
             return ids_[0]
          if type_ == "role":
@@ -51,34 +54,43 @@ class Builtins(FluxCog):
             p_dict: ty.List[str] = json.loads(target_)
             return discord.Permissions(**{p: True for p in p_dict}).value
 
-      def parse_auth_context(ctx: GuildAwareCtx, type_: str, target_: str) -> ManualAuthCtx:
+      async def parse_auth_context(ctx: GuildAwareCtx, type_: str, target_: str) -> ManualAuthCtx:
          if type_ == "user":
             ids_ = utils.find_mentions(target_)
             if not ids_:
                raise CommandError(f"No user ID found in {target_}")
 
             return ManualAuthCtx(flux=self.flux, auth_list=AuthList(user=self.flux.get_user(ids_[0]).id), config_identifier=ids_[0])
-         auth_id = parse_auth_id(ctx, type_=type_, target_=target_)
+         auth_id = await parse_auth_id(ctx, type_=type_, target_=target_)
          if type_ == "member":
-            member = ctx.guild.get_member(auth_id)
+            member = await utils.get_or_fetch_member(ctx.guild, auth_id)
             if not member:
                raise CommandError(f"No member found with id `{auth_id}`")
-            return ManualAuthCtx(flux=self.flux, auth_list=AuthList(user=member.id, roles=[r.id for r in member.roles], permissions=member.guild_permissions),
+            return ManualAuthCtx(flux=self.flux,
+                                 auth_list=AuthList(
+                                    user=member.id, roles=[r.id for r in member.roles],
+                                    permissions=member.guild_permissions),
                                  config_identifier=str(ctx.guild.id))
          if type_ == "role":
             role = ctx.guild.get_role(auth_id)
             if not role:
                raise CommandError(f"No role found with id `{auth_id}`")
-            return ManualAuthCtx(flux=self.flux, auth_list=AuthList(roles=[auth_id], permissions=role.permissions), config_identifier=str(ctx.guild.id))
+            return ManualAuthCtx(
+               flux=self.flux,
+               auth_list=AuthList(roles=[auth_id], permissions=role.permissions),
+               config_identifier=str(ctx.guild.id))
 
          if type_ == "permissions":
             try:
-               return ManualAuthCtx(flux=self.flux, auth_list=AuthList(permissions=discord.Permissions(permissions=auth_id)), config_identifier=str(ctx.guild.id))
+               return ManualAuthCtx(
+                  flux=self.flux,
+                  auth_list=AuthList(permissions=discord.Permissions(permissions=auth_id)),
+                  config_identifier=str(ctx.guild.id))
             except TypeError as e:
                raise CommandError(f"Permissions `{auth_id}` could not be parsed. See:\n{e}")
 
       # @CommandCheck.check(lambda ctx: ctx.author.id == self.flux.admin_id)
-      # @self._commandeer(name="reload", parsed=False, private=True)
+      # @self._commandeer(name="reload", private=True)
       # async def reload(ctx: MessageContext, cog_name: str):
       #    reloaded_cogs = []
       #    for cog in s.cogs:
@@ -92,18 +104,18 @@ class Builtins(FluxCog):
       #    ctx.flux.cogs = reloaded_cogs
       #    return Response()
 
-      @self._commandeer(name="asif", parsed=False, default_auths=[Record.allow_all()])
-      async def __asif(ctx: CommandCtx, args: str, ):
+      @self._commandeer(name="asif", default_auths=[Record.allow_all()])
+      async def __asif(ctx: GuildCommandCtx, args: str, ):
          """
          asif [type] <target>/{target} command args*
          ==
          Runs `command` as if it was being run by a `target` user, member, role, or permission-haver
          ==
-         [type]: [user/role/member/permissions/u/r/m/p] the type of `target`;
-         <target>: <user/member/role>. Simulates usage by a given user, member,
-         member with a role, or member that has a set of permissions. See s.ze.ax/perm for {perms} structure;
+         [type]: [user/role/member/permissions/u/r/m/p] the type of `target`
+         <target>: <user/member/role>. Simulates usage by a given caller, \\
+         with a role, or a set of permissions. See [here](s.ze.ax/perm) for {perms} names
          {target}: JSON Array of permissions to simulate having
-         command: Name of the Command to run as `target`;
+         command: Name of the Command to run as `target`
          args: command arguments to pass to the Command
          ==
          :param ctx:
@@ -112,10 +124,13 @@ class Builtins(FluxCog):
          """
 
          try:
-            mock_type, mock_target, command, command_args = utils.regex_parse(re.compile(r"(\S+)\s+(\S+)\s+((\[[^\]]*\])|([^\s\[]+))\s+(\S+)\s*(.*)"),
-                                                                              args,
-                                                                              [x - 1 for x in [1, 2, 3, 6, 7]])
-         except (ValueError, AttributeError):
+            mock_type, mock_target, command, command_args = utils.regex_parse(
+               re.compile(r"(\S+)\s+((\[[^\]]*\])|([^\s\[]+))\s+(\S+)\s*(.*)"),
+               args,
+               [x - 1 for x in [1, 2, 5, 6]]
+            )
+         except (ValueError, AttributeError) as e:
+            logger.info(e)
             raise CommandError(f"See `help asif` for usage")
 
          MOCK_TYPES = {
@@ -129,8 +144,11 @@ class Builtins(FluxCog):
             raise CommandError(f"`{mock_type}` must be in [{', '.join(MOCK_TYPES.keys())}]")
 
          cmd_name, cmd_args, *_ = [*command.split(" ", 1), None]
-         mock_auth_ctx = parse_auth_context(ctx=ctx.msg_ctx, type_=mock_type, target_=mock_target)
-         mock_author_ctx = ManualAuthorCtx(ctx.msg_ctx.message.guild.get_member(mock_target)) if ctx.msg_ctx.message.guild else ctx.author_ctx
+         mock_auth_ctx = await parse_auth_context(ctx=ctx.msg_ctx, type_=mock_type, target_=mock_target)
+         if ctx.msg_ctx.message.guild:
+            mock_author_ctx = ManualAuthorCtx(author=await utils.get_or_fetch_member(ctx.msg_ctx.guild, mock_target))
+         else:
+            mock_author_ctx = ctx.author_ctx
 
          cmd = utils.find_cmd_or_cog(self.flux, cmd_name, only="command")
 
@@ -138,14 +156,22 @@ class Builtins(FluxCog):
             raise CommandError(f"Command {cmd_name} not found")
          if Auth.accepts_all(ctx.auth_ctxs + [mock_auth_ctx], cmd):
             await self.flux.router.submit(
-               event=CommandEvent(flux=self.flux, cmd_ctx=CommandCtx(ctx.msg_ctx, mock_author_ctx, ctx.auth_ctxs + [mock_auth_ctx]), cmd_name=cmd_name, cmd_args=cmd_args))
+               event=CommandEvent(flux=self.flux,
+                                  cmd_ctx=CommandCtx(
+                                     self.flux,
+                                     ctx.msg_ctx,
+                                     mock_author_ctx,
+                                     ctx.auth_ctxs + [mock_auth_ctx]
+                                  ),
+                                  cmd_name=cmd_name,
+                                  cmd_args=cmd_args))
          else:
             raise CommandError(f"Can only mock commands that you have access to")
 
          return Response()
 
-      @self._commandeer(name="setprefix", parsed=False, default_auths=[Record.allow_perm(discord.Permissions(manage_guild=True))])
-      async def __set_prefix(ctx: CommandCtx, prefix: str):
+      @self._commandeer(name="setprefix", default_auths=[Record.allow_server_manager()])
+      async def __set_prefix(ctx: GuildCommandCtx, prefix: str):
          """
          setprefix prefix
          ==
@@ -162,7 +188,7 @@ class Builtins(FluxCog):
             cfg["prefix"] = prefix.strip()
          return Response()
 
-      @self._commandeer(name="exec", parsed=False, override_auths=[Record.deny_all()])
+      @self._commandeer(name="exec", override_auths=[Record.deny_all()])
       async def __exec(ctx: CommandCtx, script: str):
          """
          exec ute order 66
@@ -193,34 +219,38 @@ class Builtins(FluxCog):
                           f"**OUT**:\n"
                           f"```py\n{res}\n```"), trashable=True)
 
-      @self._commandeer(name="auth", parsed=False, default_auths=[Record.allow_perm(discord.Permissions(manage_guild=True))])
-      async def __auth(ctx: CommandCtx, auth_str):
+      @self._commandeer(name="auth", default_auths=[Record.allow_server_manager()])
+      async def __auth(ctx: GuildCommandCtx, auth_str):
          """
-         auth name [rule] <id>/{perm} [id_type]
+         auth name [rule] [id_type] <id>/{perm}
          ==
-         Authorizes some group (member, has a role, or has a permission) to use a command or a cog
+         Authorizes some group (a member, members that have a role, or have some permissions) to use a command or a cog
          ==
          name: Command name or Cog name;
          [rule]: [ALLOW/DENY];
-         <id>: <member/role> The target member or role to allow;
-         {perm:} A permission JSON array representing a set of permissions that a user must have ALL of.
+         id_type: [member/role/permission/m/r/p] The type that `id` is
+         <id>: <member/role> The target member or role to allow
+         {perm}: A permission JSON array representing a set of permissions that a user must have ALL of. \\
          ex. ["manage_server","kick_members"]
-         id_type: [MEMBER/ROLE/PERMISSION] Whatever `id` corresponds to
+         EXAMPLE: Want to allow the role @moderator to use `auth` (Note: This effectively gives full bot access): \\
+         `auth auth ALLOW role @moderators`
          ==
          :param ctx:
          :param auth_str:
          :return:
          """
          try:
-            rule_subject, rule, rule_target_id_raw, rule_type = utils.regex_parse(re.compile(r"(\S+)\s+(\S+)\s+((\[[^\]]*\])|([^\s\[]+))\s+(\S+)"),
-                                                                                  auth_str,
-                                                                                  [x - 1 for x in [1, 2, 3, 6]])
+            rule_subject, rule, id_type, rule_target_id_raw, = utils.regex_parse(
+               re.compile(r"(\S+)\s+(\S+)\s+(\S+)\s+((\[[^\]]*\])|([^\s\[]+))"),
+               auth_str,
+               [x - 1 for x in [1, 2, 3, 4]]
+            )
          except (ValueError, AttributeError):
             raise CommandError(f"See `help auth` for usage")
 
          rule_subject = rule_subject.lower()
          rule = rule.upper()
-         rule_type = rule_type.lower()
+         id_type = id_type.lower()
 
          cmd_or_cog = utils.find_cmd_or_cog(self.flux, rule_subject)
          if not cmd_or_cog:
@@ -228,16 +258,17 @@ class Builtins(FluxCog):
          if rule not in ["ALLOW", "DENY"]:
             raise CommandError(f'rule {rule} not in ["ALLOW","DENY"]')
          try:
-            target_id = parse_auth_id(ctx.msg_ctx, type_=self.RULETYPES[rule_type], target_=rule_target_id_raw)
+            target_id = await parse_auth_id(ctx.msg_ctx, type_=self.RULETYPES[id_type], target_=rule_target_id_raw)
          except KeyError:
-            raise CommandError(f"Rule type {rule_type} not in {self.RULETYPES.keys()}")
+            raise CommandError(f"Rule type {id_type} not in {self.RULETYPES.keys()}")
 
-         record = Record(rule=rule, target_id=target_id, target_type=rule_type.upper())
+         record = Record(rule=rule, target_id=target_id, target_type=id_type.upper())
          await Auth.add_record(ctx.msg_ctx, auth_id=cmd_or_cog.auth_id, record=record)
          return Response(f"Added record {record}")
 
-      @self._commandeer(name="userinfo", parsed=False, default_auths=[Record.allow_perm(discord.Permissions(manage_guild=True))])
-      async def __userinfo(ctx: CommandCtx, target_raw):
+      # noinspection PyPep8Naming
+      @self._commandeer(name="userinfo", default_auths=[Record.allow_server_manager()])
+      async def __userinfo(ctx: GuildCommandCtx, target_raw):
          """
          userinfo (<user/member>)
          ==
@@ -249,27 +280,29 @@ class Builtins(FluxCog):
          :param target_raw:
          :return:
          """
+         utils.perm_check(ctx.msg_ctx.channel, discord.Permissions(embed_links=True))
+
          if not target_raw:
             target = ctx.author_ctx.author
          else:
-            if not (target := utils.find_mentions(target_raw)):
+            if not (target := utils.find_mentions(target_raw)[0]):
                raise CommandError(f"Cannot find a user/member in `{target_raw}`. It should either be an ID or a mention")
 
             if isinstance(ctx.msg_ctx, GuildMessageCtx):
-               target = ctx.msg_ctx.guild.get_member(target)
+               target = await utils.get_or_fetch_member(ctx.msg_ctx.guild, target)
             else:
                target = await ctx.msg_ctx.flux.get_user(target)
 
-         embed = discord.Embed(title=f"{target}'s Userinfo",  color=target.color)
+         embed = discord.Embed(title=f"{utils.EMOJI.question}{target}'s Userinfo", color=target.color)
          embed.set_thumbnail(url=str(target.avatar_url))
          embed.add_field(name="Display Name", value=utils.copylink(target.display_name), inline=True)
          embed.add_field(name="ID", value=utils.copylink(str(target.id)), inline=False)
-         embed.add_field(name="Latest Join", value=utils.copylink(target.joined_at.strftime("%I:%M:%S %p on %a, %b %d, %Y")), inline=False)
-         embed.add_field(name="Creation Date", value=utils.copylink(target.created_at.strftime("%I:%M:%S %p on %a, %b %d, %Y")), inline=False)
+         embed.add_field(name="Latest Join", value=utils.copylink(target.joined_at.strftime(utils.DATETIME_FMT_L)), inline=False)
+         embed.add_field(name="Creation Date", value=utils.copylink(target.created_at.strftime(utils.DATETIME_FMT_L)), inline=False)
 
          if isinstance(ctx.msg_ctx, GuildMessageCtx):
             if target.color != discord.Color.default():
-               embed.add_field(name="Color", value=utils.copylink(str(target.color.value)), inline=False)
+               embed.add_field(name="Color", value=utils.copylink(hex(target.color.value).upper()), inline=False)
             if target.premium_since:
                delta = (datetime.datetime.utcnow() - target.premium_since).days
                D_IN_M = 29.53
@@ -301,8 +334,106 @@ class Builtins(FluxCog):
             )
          return Response(embed=embed)
 
-      @self._commandeer(name="help", parsed=False, default_auths=[Record.allow_all()])
-      async def __get_help(ctx: CommandCtx, help_target: ty.Optional[str]):
+      @self._commandeer(name="serverinfo", default_auths=[Record.allow_server_manager()])
+      async def __serverinfo(ctx: GuildCommandCtx, _):
+         """
+         serverinfo
+         ==
+         Gets information about the server
+         ==
+         ==
+         :param ctx:
+         :param _:
+         :return:
+         """
+         utils.perm_check(ctx.msg_ctx.channel, discord.Permissions(embed_links=True))
+
+         g = ctx.msg_ctx.guild
+         embed = discord.Embed(title=f"{utils.EMOJI.question}{g} Server Info")
+         embed.set_thumbnail(url=str(ctx.msg_ctx.guild.icon_url))
+         # Info
+         embed.add_field(name="Owner", value=f"<@!{g.owner_id}>", inline=True)
+         embed.add_field(name="Region", value=f"{g.region}", inline=True)
+         embed.add_field(name="Members", value=f"{g.member_count}")
+
+         # Creation
+         embed.add_field(name="Creation", value=g.created_at.strftime(utils.DATETIME_FMT_L), inline=False)
+
+         # Extra Info
+         embed.add_field(name="MFA Required", value=f"{bool(g.mfa_level)}", inline=True)
+         embed.add_field(name="Verify Level", value=f"{g.verification_level}", inline=True)
+         embed.add_field(name="Filter", value=f"{g.explicit_content_filter}", inline=True)
+
+         # Boosters
+         boosters_haste = await utils.haste(
+            self.flux.aiohttp_session,
+            tabulate.tabulate(
+               headers=["Member", "Mention"],
+               tabular_data=[[str(member), member.mention] for member in g.premium_subscribers])
+         ) if g.premium_subscribers else ""
+         embed.add_field(name="Nitro Boosters", value=f"[{g.premium_subscription_count} boosters]({boosters_haste})", inline=True)
+         embed.add_field(name="Boost Level", value=f"{g.premium_tier}", inline=True)
+
+         # Emoji
+         emoji_haste = await utils.haste(
+            self.flux.aiohttp_session,
+            content=tabulate.tabulate(
+               headers=["Emoji", "Animated", "URL"],
+               tabular_data=[[str(emoji), emoji.animated, emoji.url] for emoji in g.emojis]
+            )
+         )
+         embed.add_field(name="Emoji", value=f"[{len(g.emojis)}/{g.emoji_limit*2} emojis]({emoji_haste})", inline=False)
+
+         # Features
+         features_haste = await utils.haste(
+            self.flux.aiohttp_session,
+            content="\n".join(feature for feature in g.features)
+         ) if g.features else ""
+         embed.add_field(name="Features", value=f"[{len(g.features)} features enabled]({features_haste})", inline=False)
+
+         # Channels
+         text_channels = [channel for channel in g.text_channels if channel.overwrites_for(g.default_role).read_messages is not False and g.default_role.permissions.read_messages]
+         public_channels_haste = await utils.haste(
+            self.flux.aiohttp_session,
+            content=tabulate.tabulate(
+               [[f"{channel}",
+                 f"{channel.id}",
+                 f"{channel.category}",
+                 f"{channel.created_at.strftime(utils.DATETIME_FMT_S)}",
+                 f"{channel.permissions_synced}",
+                 f"{channel.slowmode_delay}s",
+                 f"{channel.position}",
+                 ]
+                for channel in text_channels],
+               headers=("Name", "ID", "Category", "Creation", "Perm Sync", "Slow", "Position")
+            )
+         ) if text_channels else ""
+         embed.add_field(name="Public Text Channels", value=f"[{len(text_channels)} Channels]({public_channels_haste})", inline=True)
+
+         voice_channels = [channel for channel in g.voice_channels if channel.overwrites_for(g.default_role).connect is not False and g.default_role.permissions.connect]
+         public_vc_haste = await utils.haste(
+            self.flux.aiohttp_session,
+            content=tabulate.tabulate(
+               [[f"{channel}",
+                 f"{channel.id}",
+                 f"{channel.category}",
+                 f"{channel.created_at.strftime(utils.DATETIME_FMT_S)}",
+                 f"{channel.permissions_synced}",
+                 f"{channel.bitrate // 1000} kbps",
+                 f"{channel.user_limit}"
+                 ]
+                for channel in voice_channels],
+               headers=("Name", "ID", "Category", "Creation", "Perm Sync", "Bitrate", "User Limit")
+
+            )
+         ) if voice_channels else ""
+
+         embed.add_field(name="Public Voice Channels", value=f"[{len(voice_channels)} Channels]({public_vc_haste})", inline=True)
+         return Response(embed=embed)
+         pass
+
+      @self._commandeer(name="help", default_auths=[Record.allow_all()])
+      async def __get_help(ctx: GuildCommandCtx, help_target: ty.Optional[str]):
          """
          help (command_name)
          ==
@@ -311,18 +442,16 @@ class Builtins(FluxCog):
          command_name: The command to get help for. Command list if not provided. Gets help about reading help if you `help help`
          ==
          :param ctx:
-         :param args:
          :param help_target: what to get help about
-         :param auth_ctx: Auth Context
          :return:
          """
-         print("HELP!")
+         utils.perm_check(ctx.msg_ctx.channel, discord.Permissions(embed_links=True))
          configs = self.flux.CONFIG.of(ctx.msg_ctx)
          authorized_cmds: ty.Dict[str, Command] = {command.name: command for cog in self.flux.cogs for command in cog.commands if
                                                    Auth.accepts_all(ctx.auth_ctxs, command) and command.name != "help"}
 
          if not help_target:
-            help_embed = discord.Embed(title=f"{utils.EMOJIS['question']} Command Help", description=f"{configs['prefix']}help <command> for more info")
+            help_embed = discord.Embed(title=f"{utils.EMOJI.question} Command Help", description=f"{configs['prefix']}help <command> for more info")
             for cmd_name, command in authorized_cmds.items():
                usage = "\n".join([f"{configs['prefix']}{usage}" for usage in command.short_usage.split("\n")])
                help_embed.add_field(name=cmd_name, value=usage, inline=False)
@@ -337,12 +466,12 @@ class Builtins(FluxCog):
             embed.add_field(name="<user>", value="Either an ID or a Mention of something. E.g. a @user", inline=False)
             embed.add_field(name="(optional)", value="Can leave this out", inline=False)
             embed.add_field(name="{json}", value="Json. No spaces please ;w;", inline=False)
-            embed.add_field(name="extra*", value="Spaces OK", inline=False)
+            embed.add_field(name="*extra", value="Can put multiple of these. Spaces okay.", inline=False)
 
             return Response(embed=embed)
 
          if help_target not in authorized_cmds:
-            return Response(f"No command `{help_target}` to show help for", errored=True)
+            return Response(f"No command `{help_target}` to show help for", status="error")
 
          cmd = authorized_cmds[help_target]
          embed = discord.Embed(
@@ -350,7 +479,7 @@ class Builtins(FluxCog):
             description=cmd.description)
 
          embed.add_field(name="Usage", value=cmd.short_usage, inline=False)
-
+         print(cmd.param_usage)
          for arg, detail in cmd.param_usage:
             embed.add_field(name=arg.strip(), value=detail.strip(), inline=False)
             # embed.add_field(name=detail,value="", inline=False)
